@@ -1,29 +1,48 @@
 import json
 
 from pathlib import Path
-from typing import List
+from typing import List, Optional
 from dataclasses import field
 
 from ..models import Assignment, Problem
 from ..shared import Files
+from .exception import CompilationException
 
 
 class CompilationProblem(Problem):
     """Add additional fields only used for build."""
 
-    number: int
+    number: Optional[int]
     path: Path
     assignment: "CompilationAssignment"
 
     percentage: float = None
 
+    @property
+    def index_path(self) -> Path:
+        return self.path.joinpath(Files.PROBLEM)
+
     @classmethod
-    def read(cls, assignment: "CompilationAssignment", reference: dict, root: Path, number: int) -> "CompilationProblem":
+    def read(
+            cls,
+            assignment: "CompilationAssignment",
+            reference: dict,
+            root: Path,
+            number: int = None) -> "CompilationProblem":
         """Load a problem from the assignment path and reference."""
 
         path = root.joinpath(reference["path"])
-        with path.joinpath(Files.PROBLEM).open() as file:
-            data = json.load(file)
+        index_path = path.joinpath(Files.PROBLEM)
+
+        try:
+            with index_path.open() as file:
+                data = json.load(file)
+        except FileNotFoundError:
+            raise CompilationException(message=f"No such path {index_path} for assignment {assignment.path}")
+        except (PermissionError, OSError):
+            raise CompilationException(message=f"Failed to read {index_path} for assignment {assignment.path}")
+        except json.decoder.JSONDecodeError:
+            raise CompilationException(message=f"Failed to deserialize {index_path} for assignment {assignment.path}")
 
         data["short"] = reference.get("short", data.get("short", path.parts[-1]))
         data["relative_path"] = reference.get("relative_path", data["short"])
@@ -70,12 +89,25 @@ class CompilationAssignment(Assignment):
     problems: List[CompilationProblem]
     path: Path = field(init=False)
 
+    @property
+    def index_path(self) -> Path:
+        return self.path.joinpath(Files.ASSIGNMENT)
+
     @classmethod
     def read(cls, path: Path) -> "CompilationAssignment":
         """Load an assignment from a containing directory."""
 
-        with path.joinpath(Files.ASSIGNMENT).open() as file:
-            data = json.load(file)
+        index_path = path.joinpath(Files.ASSIGNMENT)
+
+        try:
+            with index_path.open() as file:
+                data = json.load(file)
+        except FileNotFoundError:
+            raise CompilationException(message=f"No such path {index_path}")
+        except (PermissionError, OSError):
+            raise CompilationException(message=f"Failed to read {index_path}")
+        except json.decoder.JSONDecodeError:
+            raise CompilationException(message=f"Failed to deserialize {index_path}")
 
         data["short"] = data.get("short", path.parts[-1])
         self = cls.load(data, problems=[])
@@ -83,12 +115,11 @@ class CompilationAssignment(Assignment):
         counter = 1
         total_weight = 0
         for reference in data.pop("problems"):
-            number = None
-            if any(filter(None, reference["grading"])):
-                number = counter
+            problem = CompilationProblem.read(self, reference, path)
+            if problem.grading.is_automated or problem.grading.is_review or problem.grading.is_manual:
+                problem.number = counter
                 counter += 1
 
-            problem = CompilationProblem.read(self, reference, path, number)
             total_weight += problem.grading.weight
             self.problems.append(problem)
 
